/****************************************************************************
** ik220_nice.c
****************************************************************************
*/

#define EXTERN extern

/*------------------------
** Include files
**------------------------
*/
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <sys/ioctl.h>

#include <ik220.h>

/*-------------------------------------------------------------------------
** IK220_get_naxis() - returns the number of axis found by the driver
**-------------------------------------------------------------------------
*/
int IK220_get_naxis( int Fd, int * num_axis )
{
   struct ik220_parms_t p;

   p.axis = 0;       // n/a 
   p.l[0] = 0;
   ioctl( Fd, IK220_IOCTL_GET_NAXIS, &p );

   *num_axis = p.l[0];
   return p.ret;
}  
/*-------------------------------------------------------------------------
** IK220_download() - function to download program code to G28 chip on IK220 card
**-------------------------------------------------------------------------
*/
int IK220_download( int Fd, int axis, unsigned short *data, int nwords )
{
   struct ik220_parms_t p;
                                                                                                   
   p.axis = axis;            // axis index
   p.p[0] = data;            // get address of the array of unsigned shorts
   p.l[0] = nwords;          // number of word in data[]
                                                                                                   
   ioctl( Fd, IK220_IOCTL_DOWNLOAD, &p );

   return p.ret;
}  
 
/*-------------------------------------------------------------------------
** IK220_getver() -
**-------------------------------------------------------------------------
*/
int IK220_getver( int Fd, int axis, char * buf )
{
   struct ik220_parms_t p;

   *buf = 0;            // set to null
   p.axis = axis;       // axis index
   p.p[0] = buf;        // return version string here.

   ioctl( Fd, IK220_IOCTL_GETVER, &p );
   return p.ret;
}  

/*-------------------------------------------------------------------------
** IK220_led() - set the LED to 0 (off), 1(on), 2(blink)
**-------------------------------------------------------------------------
*/
int IK220_led( int Fd, int axis, int mode )
{
   struct ik220_parms_t p;

   p.axis = axis;            // axis index
   p.l[0] = mode;            // mode
   ioctl( Fd, IK220_IOCTL_LED, &p );
   return p.ret;
}  

/*-------------------------------------------------------------------------
** IK220_read48() -
**-------------------------------------------------------------------------
*/
int IK220_read48( int Fd, int axis, int latch, ulong *w1, ulong *w2, long long * R_pos )
{
   struct ik220_parms_t p;

   long long pos;
   ulong *lp;

   p.axis = axis;            // axis index
   p.l[0] = latch;
   p.l[1] = 0;
   p.l[2] = 0;
   ioctl( Fd, IK220_IOCTL_READ48, &p );
   *w1 = (ulong) p.l[1];
   *w2 = (ulong) p.l[2];


   lp = (ulong*) &pos;
   lp[0] = *w1;
   lp[1] = *w2;
   if( lp[1] & 0x00008000 ) lp[1] |= 0xffff0000;   // extend sign bits to MSB
   *R_pos = pos;

   return p.ret;
}  

/*-------------------------------------------------------------------------
** IK220_start() - start counter 
**-------------------------------------------------------------------------
*/
int IK220_start( int Fd, int axis )
{
   struct ik220_parms_t p;

   p.axis = axis;            // axis index
   ioctl( Fd, IK220_IOCTL_START, &p );
   return p.ret;
}  

/*-------------------------------------------------------------------------
** IK220_t1() -
**-------------------------------------------------------------------------
*/
int IK220_t1( int Fd, int axis )
{
   struct ik220_parms_t p;

   p.axis = axis;            // axis index
   ioctl( Fd, IK220_IOCTL_T1, &p );
   return p.ret;
}  
