/*----------------------------------------------------------------
 * ik220_prv.h - Defines just for driver module
 *----------------------------------------------------------------
 */

#ifndef _IK220_PRV_H
#define _IK220_PRV_H


#define IK220_VERSION     "04.01"  // Version is YY.MM
#define IK220_MAJOR       155      // major device number

#define IK220_PCI_DEVICE_ID 0x9050  // vendor ID for locating PCI board
#define IK220_PCI_VENDOR_ID 0x10b5  // Devicd ID for locating PCI board

#define MAX_CARDS         4         // Number of IK220 supported
#define MAX_AXIS          8         // Number of Axis supported (2 per card)

#define IO_TIMEOUT_2ms  400   // IO tiemout loop. Total is 2 ms (400*5)
#define IO_TIMEOUT_1ms  200   // IO tiemout loop. Total is 1 ms (200*5).
#define IO_UDELAYS        5   // 5 mico seconds

//
// G28 Register read/write defines (Byte offsets)
//
#define G28_Port0        0     // Address of port  0
#define G28_Port1        2     // Address of port  1
#define G28_Port2        4     // Address of port  2
#define G28_Port3        6     // Address of port  3
#define G28_Port4        8     // Address of port  4
#define G28_Port5       10     // Address of port  5
#define G28_Port6       12     // Address of port  6
#define G28_Port7       14     // Address of port  7
#define G28_Port8       16     // Address of port  8
#define G28_Port9       18    // Address of port  9
#define G28_Port10      20     // Address of port 10
// write register
#define G28_SetFlag0Reg    22     // Address of set flag 0 register
#define G28_ControlReg     24     // Address of control register
// read register
#define G28_ClrFlag1Reg    22     // Address of clear flag 1 register
#define G28_StatusReg      24     // Address of status register
#define G28_Flag0Reg       26     // Address of flag 0 register
#define G28_Flag1Reg       28     // Address of flag 1 register
#define G28_CodeReg        30     // Address of code register

//
// G28 command register
//
#define WaitStates  0x0000                // no wait states
#define G28C_RunMode      (0x0000 | WaitStates)    // CPU run         mode
#define G28C_BootMode     (0x0001 | WaitStates)    // CPU boot        mode
#define G28C_SingleMode   (0x0002 | WaitStates)    // CPU single step mode
#define G28C_DebugMode    (0x0003 | WaitStates)    // CPU debug      mode
#define G28C_ReadRamMode  (0x0004 | G28C_BootMode)      // CPU boot        mode and read  ram
#define G28C_WriteRamMode (0x0008 | G28C_BootMode)      // CPU boot        mode and write ram
#define G28C_DoStepMode   (0x0004 | G28C_SingleMode)    // CPU single step mode and step
#define G28C_ReadRegs     (0x0004 | G28C_DebugMode)     // CPU debug      mode and read  register
#define G28C_WriteRegs    (0x0008 | G28C_DebugMode)     // CPU debug      mode and write register
//
// G28 Status Register returns
//
#define G28_PipeEmpty     0x0002             // CPU pipeline empty
//
// G28 semaphores
//
#define G28Sem0         0x0001          // Semaphore  0 from G28
#define G28Sem1         0x0002          // Semaphore  1 from G28
#define G28Sem2         0x0004          // Semaphore  2 from G28
#define G28Sem3         0x0008          // Semaphore  3 from G28
#define G28Sem4         0x0010          // Semaphore  4 from G28
#define G28Sem5         0x0020          // Semaphore  5 from G28
#define G28Sem6         0x0040          // Semaphore  6 from G28
#define G28Sem7         0x0080          // Semaphore  7 from G28
#define G28Sem8         0x0100          // Semaphore  8 from G28
#define G28Sem9         0x0200          // Semaphore  9 from G28
#define G28Sem10        0x0400          // Semaphore 10 from G28

//
// IK220 rename some of these ports
//
#define CmdPort   G28_Port0
#define DatPort0  G28_Port0
#define DatPort1  G28_Port1
#define DatPort2  G28_Port2
#define DatPort3  G28_Port3
#define DatPort4  G28_Port4
#define DatPort5  G28_Port5
#define DatPort6  G28_Port6
#define DatPort7  G28_Port7
#define DatPort8  G28_Port8
#define DatPort9  G28_Port9
#define StaPort   G28_Port10

//
// IK220 commands
//
#define cmdStart           0x0102    // Command: start counter
#define cmdReadCnt0        0x0301    // Command: read counter 0 value
#define cmdGetVersion      0x0407    // Command: get software version
#define cmdLedOff          0x1010    // Command: led off
#define cmdLedOn           0x1011    // Command: led on
#define cmdLedFlash        0x1012    // Command: led flash

#endif  // _IK220_PRV_H

