
#define EXTERN
#define MAIN     1
 
/*--------------------------
**  include files
**--------------------------
*/
#include <sys/types.h>
#include <sys/time.h>
#include <ctype.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <errno.h>
#include <rpc/rpc.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/ioctl.h>

#include <semaphore.h>
//#include <mqueue.h>

#include <ik220.h>
#include <ik220_pgm.h>

/*-----------------------------
** helpful defines 
**-----------------------------
*/
#define INRANGE(a,x,b)   ((a) <= (x) && (x) <= (b))

#define ERR_NONE                    (0)    /* No error        */
#define ERR_INV_KW                 (-1)    /* Invalid keywork */
#define ERR_INV_RNG                (-2)    /* Invalid Range   */
#define ERR_INV_FORMAT             (-3)    /* Invalid Format or syntax error */

/*---------------------------
** function prototypes
**---------------------------
*/
int cmd_execute( char * cmd_buf);

int do_download( char *par );
int do_getver( char *par );
int do_led( char *par );
int do_pshow( char *par );
int do_start( char *par );
int do_t1( char *par );
int do_help( char *par );
int do_quit( char *par );

ulong get_ulong( char *prompt, ulong cur_val, ulong min, ulong max );
long  get_long ( char *prompt, long cur_val, long min, long max );
double get_dbl( char *prompt, double cur_val, double min, double max );

char *unpad( char *s, int c);
char *strxcpy( char *dest, const char *src, int maxlen);
int parseInt_r( int * ip, char *buf, const char *tok, char **st_ptr );
int parseIntR_r( int * ip, char *buf, const char *tok, char **st_ptr, int min, int max );
int my_atol( char *s, long *l );

/*-------------------------------------
*  Global variables
*-------------------------------------
*/
 
int  Num_axis;
int  Quit;
int  Fd;

struct PARSE_TABLE
{
  char * kw;          /* The opcode string  */
  int (* pfi)();      /* pfi is pointer to function returning an integer */
   char * desc;       /* command description */
};
 
struct PARSE_TABLE parse_table[] =
{
 
   { "Download",         do_download,        "Download - download G28 software to IK220" },
   { "getver",           do_getver,          "GetVer - Get Card/Driver Version" },
   { "led",              do_led,             "Led - set LED mode" },
   { "pshow",            do_pshow,           "pshow - display position info" },
   { "start",            do_start,           "start - start counter" },
   { "t1",               do_t1,              "T1 - Test1" },
   { "help",             do_help,            "Help   - List commands" },
   { "Quit",             do_quit,            "Quit   - quits app" },
};
 
int num_parse_item = { sizeof(parse_table) / sizeof(struct PARSE_TABLE) };


/*---------------------------------
** main()
**---------------------------------
*/
int main( int argc, char * argv[])
{
  int i, err;
  extern char *optarg;
  extern int optind;
  char buf[80];
 
  /* parse options */
  err = 0;
  while((i = getopt(argc, argv, "")) != EOF)
  {
      switch(i)
      {
         case '?':
         err++;
         break;
      }
   }

   if( err )
	{
		fprintf(stderr, "Usage: %s [] \n", argv[0]);
		exit(0);
	}
 
   /*--------------------------------------
   **  open device driver 
   */
   if( (Fd = open("/dev/ik220", O_RDWR, 0x666)) < 0)
   {
      printf("Error opening Heidenhain IK220 Counter Card device driver \n");
      exit(0);
   }
   IK220_get_naxis( Fd, &Num_axis );
   printf("Opened /dev/ik220 fd=%d Num_axis=%d \n", Fd, Num_axis );

   /* main loop for commands */
   Quit = FALSE;
   while( !Quit )
   {
      /* Get command */
      printf("Enter Command: ");
      gets( buf );
 
      cmd_execute( buf );
   }

   close( Fd );
	return 0;
}

/*-------------------------------------------------------------
** cmd_execute() - Executes a command line instruction
**-------------------------------------------------------------
*/
int cmd_execute( cmd_buf )
   char * cmd_buf;
{
   char copy_buf[80];
   int error,
      found,
      i;
 
   char * kw,
        * par;
   /*
   **  Search table for keyword
   */
   strxcpy( copy_buf, cmd_buf, sizeof(copy_buf));
 
   kw  = strtok(copy_buf, " ");
   if( (kw == NULL) || (*kw=='#') )
      return( 0 );

   if( (par = strtok( (char*)NULL, "")) != NULL )
		unpad( par, ' ' );
   else
      par = "";

   /*
   **  Search table for keyword. If found call function.
   */
   error = ERR_INV_KW;
   for(i=0, found = FALSE; i<num_parse_item && found == FALSE; )
   {
      if( !strcasecmp( kw, parse_table[i].kw ) )
      {
         found = TRUE;
         error = (parse_table[i].pfi)(par);
      }
      else
         i++;
   }
   /*
   **  Provide user feedback
   */
 
   if( error != 0 )
   {
      printf("%s - Err=%d\n", cmd_buf, error );
   }
 
   return( error );
}

/********************************************************************
**                      COMMAND FUNCTIONS                          **
********************************************************************/

/*----------------------------------------------------------------------
**  do_download - download axis software to G28 chip on IK220           
**----------------------------------------------------------------------
*/
int do_download( par )
   char * par;
{
   int rc, 
       axis,
       nwords;
   char *str_ptr;
   
   /* number of axis */
   rc = parseIntR_r( &axis, par, " ", &str_ptr, 0, Num_axis-1  );
   if( rc != ERR_NONE)
      axis = 0;

   nwords = sizeof( Pgm220 ) / sizeof(short);
   rc = IK220_download( Fd, axis, Pgm220, nwords );
   printf("Axis %d downloaded.  rc is %d.\n", axis, rc );

   return 0;
}

/*----------------------------------------------------------------------
**  do_getver - get version information from card                     
**----------------------------------------------------------------------
*/
int do_getver( par )
   char * par;
{
   int axis,
       rc;
   char buf[40];
   char *str_ptr;

   /* axis */
   rc = parseIntR_r( &axis, par, " ", &str_ptr, 0, Num_axis-1  );
   if( rc != ERR_NONE)
      axis = 0;

   rc = IK220_getver( Fd, axis, buf );
   printf("axis %d Version=[%s]  rc=%d\n", axis, buf, rc );

   return 0;
}

/*----------------------------------------------------------------------
**  do_led - set LED to 0(off), 1 (on), 2(blink)                   
**     syntax: led axis mode
**----------------------------------------------------------------------
*/
int do_led( par )
   char * par;
{
   int axis,
       rc;
   int mode;
   char * str_ptr;

   rc = parseIntR_r( &axis, par, " ", &str_ptr, 0, Num_axis-1  );
   if( rc != ERR_NONE)
      axis = 0;

   rc = parseInt_r( &mode, NULL, " ", &str_ptr  );
   if( (rc != ERR_NONE) || !INRANGE(0, mode, 2))
      mode = 0;

   printf("setting LED on axis %d to %d \n", axis, mode);
   rc = IK220_led( Fd, axis, mode );
   return rc;
}

/*----------------------------------------------------------------------
**  do_t1 - return status          
**----------------------------------------------------------------------
*/
int do_t1( par )
   char * par;
{
   int rc;
   
   rc = IK220_t1( Fd, 0 ); // return component ID for now

   return 0;
}

/*----------------------------------------------------------------------
**  do_pshow() - Loops and shows the position of an axis. If axis == Num_axis,
**     then all axises are displayed.
**     Syntax: pshow loops axis
**----------------------------------------------------------------------
*/
int do_pshow( par )
   char * par;
{
   int rc, 
       axis,
       axis_to_show,
       loops, i;
   char * str_ptr;
   ulong l1, l2;
   long long pos;

   /* get star index */
   rc = parseInt_r( &loops, par, " ", &str_ptr  );
   if( rc != ERR_NONE)
      loops = 1;

   rc = parseInt_r( &axis_to_show, NULL, " ", &str_ptr  );
   if( (rc != ERR_NONE) || !INRANGE(0, axis_to_show, Num_axis))
      axis_to_show = 0;

   for( i=0; i<loops; i++ )
   {
      printf("\033[H");   /* home */
      if( (i % 10) == 0) printf("\033[2J");  /* clear screen */
      printf(" loops %d / %d   Axis %d\n\n", i+1, loops, axis_to_show);
   
      printf("Axis 32bit_pos  RegData0    RegData1    48bit_hex_work     48bit_position \n");

      for( axis=0; axis<Num_axis; axis++ )
      {
         if( (axis == axis_to_show ) || (axis_to_show==Num_axis) )
         {
            IK220_read48( Fd, axis, 0, &l1, &l2, &pos );
            printf(" %02d  %8lld  0x%08lx  0x%08lx  0x%016llx  %lld \n", 
               axis, pos >> 16, l1, l2, pos, pos);
         }
      }

      usleep( 100000 );
   }

   return ERR_NONE;
}

/*----------------------------------------------------------------------
**  do_start - call ik220_start
**     Syntax: start axis
**----------------------------------------------------------------------
*/
int do_start( par )
   char * par;
{
   int axis,
       rc;
   char *str_ptr;

   /* axis */
   rc = parseIntR_r( &axis, par, " ", &str_ptr, 0, Num_axis-1  );
   if( rc != ERR_NONE)
      axis = 0;

   rc = IK220_start( Fd, axis ); // start the axis 
   printf("ik220_start on axis %d returns %d \n", axis, rc);

   return 0;
}

/*----------------------------------------------------------------------
**  do_help() - set Quit flag to TRUE.
**----------------------------------------------------------------------
*/
int do_help( par )
   char * par;
{
   int i;

   printf("\n"
          "This program communicates with the Adlink PCI-7248 DIO driver. \n"
          "Summary of commands:\n");

   for( i=0; i<num_parse_item; i++)
   {
      printf("   %s\n", parse_table[i].desc);
   }
   printf("\n");
   return 0;
}


/*----------------------------------------------------------------------
**  do_quit() - set Quit flag to TRUE.
**----------------------------------------------------------------------
*/
int do_quit( par )
   char * par;
{
   Quit = TRUE;
   return 0;
}

/*----------------------------------------------------------------------
**  get_long() - prompt the user and reads a long from the keyboard.
**----------------------------------------------------------------------
*/
long get_long (  char *prompt, long cur_val, long min, long max )
{
   char buf[80];

   while( 1 )
   {
      printf( "%s (%ld): ", prompt, cur_val );
      scanf("%s", buf );
      unpad( buf, ' ' );
      if( strlen(buf) >= 1 )
         cur_val = atol(buf);

      if( INRANGE(min, cur_val, max))
      {
         return cur_val;
      }
      else
         printf("\007INVALID Input. Range is %ld to %ld.\n", min, max);
   }
}


/*----------------------------------------------------------
** unpad( s ) - removes beginning and trailing characters c
**              from string s. Return s.
**----------------------------------------------------------
*/
char *unpad ( s, c )
   char * s;
   int c;
{
   int i, l;
   if( s==NULL)
      return(NULL); 
   i = l = strlen(s);
   i--;
   while ( (i>=0) && (s[i]==c) )
      s[i--] = 0;
   for( i=0; s[i]==c; i++); 
   if( i > 0 )
       memmove(s, s+i, l-i+1);
   return(s);
}

/*---------------------------------------------------------
**  strxcpy() - similar to strncpy, but will always
**       place an null character at the end of dest.
**---------------------------------------------------------
*/
char * strxcpy
(  
   char * dest,
   const char * src,
   int  maxlen
)  
{     
   strncpy( dest, src, maxlen );
   dest[maxlen-1] = '\0';
   return dest;
}     

/*----------------------------------------------------------
**  parseInt_r() - parse & convert to int.
**----------------------------------------------------------
*/
int parseInt_r( int * ip, char *buf, const char *tok, char **st_ptr )
{
   char * c; 
   long l;

   if( (c = strtok_r( buf, tok, st_ptr) ) == NULL)
      return ERR_INV_FORMAT;

   if( -1 == my_atol( c, &l))
      return ERR_INV_FORMAT;

   *ip = (int)l;
   return ERR_NONE;
}

/*-------------------------------------------------------------
** my_atol() - convert a string to a long. The string s
**   must consist of  '[white space][+-] digits [white spaces]'
**   returns error: 0 = noerror;
**                 -1 = invalid char in string
**-------------------------------------------------------------
*/

int my_atol( char *s, long *l )
{
   char *start;
   int  cnt;

   if( s == NULL)
      return -1;

   while( isspace( (int) *s) ) s++;     /* Skip leading spaces */
   start = s;

   if( *s=='-' || *s=='+' ) s++; /* First char can be a signed */

   cnt = 0;
   while( isdigit( (int)*s ) )          /* Only allow decimal digits */
   {
     s++;
     cnt++;
   }
   if( !cnt )  return -1;
                                 /* Allow ONLY trailing white spaces */
   while( *s )
      if( isspace( (int)*s ) )
         s++;
      else
         return -1;

   *l = strtol( start, (char**) NULL, 10 );
   return 0;
}

/*----------------------------------------------------------------
**  parseIntR_r() - parse & convert to int, with range checking
**---------------------------------------------------------------
*/
int parseIntR_r( int * ip, char *buf, const char *tok, char **st_ptr, int min, int max )
{
   int rc;

   if( (rc = parseInt_r( ip, buf, tok, st_ptr )) != ERR_NONE )
      return rc;

   if( !INRANGE( min, *ip, max ))
      return ERR_INV_RNG;

   return ERR_NONE;
}

